import os
import re
import sys
import webbrowser

from qgis.PyQt.QtCore import QCoreApplication, QFileSystemWatcher, QSettings, Qt
from qgis.PyQt.QtGui import QIcon, QPixmap
from qgis.PyQt.QtWidgets import QAction, QApplication, QDialog, QMenu, QSplashScreen, qApp
from qgis.PyQt.uic import loadUi
from qgis.utils import iface

from rana_qgis_customisations.ui import resources_rc


def reload_style(path):
    # Some applications will remove a file and rewrite it.  QFileSystemWatcher will
    # ignore the file if the file handle goes away so we have to keep adding it.
    watch.removePaths(watch.files())
    watch.addPath(path)
    with open(path, "r") as f:
        stylesheet = f.read()
        # Update the image paths to use full paths. Fixes image loading in styles
        path = os.path.dirname(path).replace("\\", "/")
        stylesheet = re.sub(r"url\((.*?)\)", r'url("{}/\1")'.format(path), stylesheet)
        QApplication.instance().setStyleSheet(stylesheet)


watch = QFileSystemWatcher()
watch.fileChanged.connect(reload_style)


class AboutRanaDialog(QDialog):
    def __init__(self, parent):
        super(AboutRanaDialog, self).__init__(parent)
        ui_fn = os.path.join(os.path.dirname(__file__), "ui", "about_rana_dialog.ui")
        loadUi(ui_fn, self)


class RanaQgisCustomisations:
    def __init__(self, iface):
        self.iface = iface
        self.plugin_dir = os.path.dirname(__file__)

        self.windowTitle = "Rana Desktop Client - Powered by QGIS"

        self.app = QApplication.instance()
        self.QApp = QCoreApplication.instance()

        if self.QApp is None:
            self.QApp = QApplication(sys.argv)

        self.QApp.startingUp()
        self.QApp.processEvents()
        self.app.startDragTime()

        self.iface.initializationCompleted.connect(self.customization)
        qApp.processEvents()

        self.applyStyle()

    def initGui(self):
        QSettings().setValue("/qgis/hideSplash", True)
        qApp.processEvents()

        icon = QIcon(":/rana/images/logo.png")
        self.app.setWindowIcon(icon)
        self.iface.mainWindow().setWindowIcon(icon)
        self.iface.mainWindow().setWindowTitle(self.windowTitle)

        qApp.processEvents()

        if not self.iface.mainWindow().isVisible():
            self.splash_pix = QPixmap(":/rana/images/splash.png")
            self.splash = QSplashScreen(self.splash_pix, Qt.WindowStaysOnTopHint)
            self.splash = QSplashScreen(self.splash_pix)
            self.splash.setMask(self.splash_pix.mask())
            self.splash.show()
            qApp.processEvents()
        self.applyStyle()
        self.addHelpMenuItem()
        return

    def run(self):
        pass

    def unload(self):
        qApp.processEvents()
        self.iface.initializationCompleted.disconnect(self.customization)
        self.helpAction.deleteLater()
        return

    def customization(self):
        self.splash.finish(self.iface.mainWindow())
        self.iface.mainWindow().setWindowTitle(self.windowTitle)
        qApp.processEvents()
        self.applyStyle()

    def applyStyle(self):
        path = os.path.abspath(os.path.join(self.plugin_dir, "stylesheet.qss"))
        watch.removePaths(watch.files())
        reload_style(path)

    @staticmethod
    def openRanaHelp():
        webbrowser.open_new("https://test.ranawaterintelligence.com")

    @staticmethod
    def about_rana_dialog():
        dialog = AboutRanaDialog(iface.mainWindow())
        dialog.exec_()

    def find_rana_menu(self):
        for i, action in enumerate(self.iface.mainWindow().menuBar().actions()):
            if action.menu().objectName() == "rana":
                return action.menu()
        return None

    def addHelpMenuItem(self):
        menu = self.find_rana_menu()
        if not menu:
            menu = QMenu("Rana", self.iface.mainWindow().menuBar())
            menu.setObjectName("rana")
            self.iface.mainWindow().menuBar().addMenu(menu)

        self.helpAction = QAction(
            QIcon(":/rana/images/icon.svg"),
            "Documentation",
            self.iface.mainWindow(),
        )
        self.helpAction.triggered.connect(self.openRanaHelp)
        self.helpAction.setWhatsThis("Rana Documentation")

        menu.addAction(self.helpAction)

        about_action = QAction(
            QIcon(":/rana/images/icon.svg"),
            "About Rana Desktop Client",
            self.iface.mainWindow(),
        )
        about_action.triggered.connect(self.about_rana_dialog)
        about_action.setWhatsThis("About Rana Desktop Client")

        menu.addAction(about_action)
